from datetime import datetime
import yaml
from src.data.copyData import copy_images_with_exclusion
from src.data.modelData import get_test_data, get_train_data, define_augmentation_rules
from src.enum.activierungsfunktionEnum import Aktivierungsfunktion
from src.enum.marktEnum import Markt
from src.enum.optimierungsverfahren import Optimierungsverfahren
from src.knn.configKNN import ConfigKNN
from src.knn.createKNN import evaluate_model, fit_model
from src.knn.defineKNN import define_model
from src.result.createExcelFile import create_excel_result, get_excel_workbook, get_excel_worksheet, save_excel, \
    average_evaluate_cross_validation
from src.result.customCallback import CustomCallback


def run_cross_validation():
    print("Lese Configurationsdatei")
    # Lesen der Configurationdatei
    with open("ressources/config/config.yaml", "r") as file:
        config = yaml.load(file, Loader=yaml.FullLoader)

    # Initialisieren der KNN-Modellbau Eigenschaften
    for optimization_method in Optimierungsverfahren:
        if optimization_method.name in config['knn']['exception_optimization_method']:
            continue
        for activation_function_2 in Aktivierungsfunktion:
            if activation_function_2.name in config['knn']['exception_activation_funktion_2']:
                continue
            for activation_function_128 in Aktivierungsfunktion:
                if check_if_kombination_is_not_allowed(
                        optimization_method.name, activation_function_128.name, activation_function_2.name, config
                ):
                    continue
                model_evaluate_metrics = []
                training_duration_models = []
                workbook = get_excel_workbook()
                worksheet = get_excel_worksheet(workbook)
                r = 1
                config_knn = ConfigKNN(
                    excluded_folder=Markt.Kein_Markt,
                    activation_function_2_units=activation_function_2,
                    activation_function_128_units=activation_function_128,
                    optimization_method=optimization_method
                )
                for excluded_folder in Markt:
                    if excluded_folder == Markt.Kein_Markt:
                        continue
                    config_knn.excluded_folder = excluded_folder
                    print("Konfiguration KNN: " + config_knn.__str__())
                    worksheet, r, evaluate_metrics, training_duration_model = run_model(config, config_knn, worksheet, r)
                    model_evaluate_metrics.append(evaluate_metrics)
                    training_duration_models.append(training_duration_model)
                print("Berechne den Durchschnitt der Cross-Validation beim Modellbau")
                average_evaluate_cross_validation(worksheet, r, model_evaluate_metrics, training_duration_models)
                print("Speichere die Excel Tabelle")
                save_excel(workbook, config, config_knn)


# run a model definition
def run_model(config, config_knn, worksheet, r):
    print("Initialisiere Callback")
    # Erstellen Sie eine Instanz des benutzerdefinierten Callbacks
    callback = CustomCallback()

    print("Generiere Ordner mit OOS und !OOS Daten")
    # Erstellen der Ordner mit Inhalt für die Trainings-/Testdaten und Evaluationsdaten, die für das nächste Modell
    # benötigt werden
    copy_images_with_exclusion(config["bilder"]["original_path"],
                               config["bilder"]["knn_path"],
                               config_knn.excluded_folder)

    print("Starte Zeit")
    start_time = datetime.now()
    # define model
    print("Definiere KNN Modell")
    model = define_model(config_knn)
    # create data generator
    print("Initialisiere ImageDataGenerator")
    datagen = define_augmentation_rules()
    # prepare iterator
    print("Definiere Trainings- und Testdaten Set")
    train_it = get_train_data(datagen, config["bilder"]["knn_path"])
    test_it = get_test_data(datagen, config["bilder"]["knn_path"])
    print("Trainiere das Modell")
    # fit model
    fit_model(config, model, train_it, test_it, callback)
    # evaluate model
    print("Evaluiere das Modell")
    evaluate_metrics = evaluate_model(model, test_it, datagen, config, config_knn)
    # print duration
    print("Stoppe die Zeit")
    end_time = datetime.now()
    training_duration_model = end_time - start_time
    training_duration_model_str = format(training_duration_model)
    print('Die Dauer für das Erstellen des Modells beträgt: ' + training_duration_model_str)
    # learning curves
    # print("Erstelle den Plot Graphen")
    # plot_values(history, config)
    print("Inhalte in die Excel-Tabelle schreiben")
    r = create_excel_result(worksheet, r, callback, config, config_knn, evaluate_metrics,
                                       training_duration_model)
    return r, evaluate_metrics, training_duration_model


def check_if_kombination_is_not_allowed(opt, act128, act2, config):
    currentKombination = [opt, act128, act2]
    ignoreKombinationList = config['knn']['ignoreKnnCombinations']
    if currentKombination in ignoreKombinationList:
        return True
    return False


# entry point
run_cross_validation()